==> local-booter-22.0.0-1229.1344-sources.jar: com/xebialabs/xlplatform/synthetic/xml/XmlUtils.java <==
package com.xebialabs.xlplatform.synthetic.xml;

import java.util.Iterator;
import java.util.NoSuchElementException;
import java.util.Optional;
import java.util.function.Predicate;
import org.w3c.dom.Element;
import org.w3c.dom.NodeList;

import com.xebialabs.deployit.plugin.api.reflect.Type;

public class XmlUtils {

    private XmlUtils() {
    }

    public static String getRequiredStringAttribute(Element element, String attributeName) {
        if (element.hasAttribute(attributeName)) {
            return element.getAttribute(attributeName);
        }
        throw new IllegalArgumentException("Attribute " + attributeName + " not provided");
    }

    public static Type getRequiredTypeAttribute(Element element, String attributeName) {
        Type type = getOptionalTypeAttribute(element, attributeName);
        if (type == null) {
            throw new IllegalArgumentException("Attribute " + attributeName + " not provided");
        }
        return type;
    }

    public static Type getOptionalTypeAttribute(Element element, String attributeName, Type defaultValue) {
        String typeAttr = getOptionalStringAttribute(element, attributeName, null);
        Type type = defaultValue;
        if (typeAttr != null) {
            type = Type.valueOf(typeAttr);
        }
        return type;
    }

    public static Type getOptionalTypeAttribute(Element element, String attributeName) {
        return getOptionalTypeAttribute(element, attributeName, null);
    }

    public static Optional<Type> getOptionalType(Element element, String attributeName) {
        return getOptionalString(element, attributeName).map(Type::valueOf);
    }

    public static boolean getOptionalBooleanAttribute(Element element, String attributeName, boolean defaultValue) {
        return getOptionalBoolean(element, attributeName).orElse(defaultValue);
    }

    public static Optional<Boolean> getOptionalBoolean(Element element, String attributeName) {
        return getOptionalString(element, attributeName).map(Boolean::valueOf);
    }

    public static String getOptionalStringAttribute(Element element, String attributeName, String defaultValue) {
        return getOptionalString(element, attributeName).orElse(defaultValue);
    }

    public static Optional<String> getOptionalString(Element element, String attributeName) {
        if (element.hasAttribute(attributeName)) {
            return Optional.ofNullable(element.getAttribute(attributeName));
        } else {
            return Optional.empty();
        }
    }

    public static Optional<String> getOptionalTextOfChild(Element element, String childElementName) {
        Iterator<Element> children = childrenByName(element, childElementName::equals).iterator();
        if (children.hasNext()) {
            return Optional.ofNullable(children.next().getFirstChild().getTextContent());
        }
        return Optional.empty();
    }

    public static Iterable<Element> childrenByName(final Element element, final Predicate<String> matcher) {
        return () -> childByName(element, matcher);
    }

    public static Iterator<Element> childByName(final Element element, final Predicate<String> matcher) {
        return new Iterator<Element>() {
            private int i = -1;
            private NodeList childNodes = element.getChildNodes();
            private int nextIndex = -1;

            @Override
            public boolean hasNext() {
                if (nextIndex == i) nextIndex = findNext();
                return nextIndex > i;
            }

            @Override
            public Element next() {
                if (nextIndex == i) nextIndex = findNext();
                if (nextIndex < i) throw new NoSuchElementException("There are no more matching elements");
                i = nextIndex;

                return (Element) childNodes.item(i);
            }

            private int findNext() {
                int next = i;
                while (next < childNodes.getLength()) {
                    next++;
                    if (childNodes.item(next) instanceof Element) {
                        Element e = (Element) childNodes.item(next);
                        if (matcher.test(e.getNodeName())) {
                            return next;
                        }
                    }
                }
                return i - 1;
            }

            @Override
            public void remove() {
                throw new UnsupportedOperationException();
            }
        };
    }

    public static <E> void forEach(Iterator<E> iterator, Closure<E> closure) {
        while (iterator.hasNext()) {
            closure.call(iterator.next());
        }
    }

    public interface Closure<E> {
        void call(E e);
    }
}

==> local-booter-23.1.0-404.1515-sources.jar: com/xebialabs/xlplatform/synthetic/xml/XmlUtils.java <==
package com.xebialabs.xlplatform.synthetic.xml;

import java.util.Iterator;
import java.util.NoSuchElementException;
import java.util.Optional;
import java.util.function.Predicate;

import com.xebialabs.xlplatform.synthetic.TypeName;
import org.w3c.dom.Element;
import org.w3c.dom.NodeList;

public class XmlUtils {

    private XmlUtils() {
    }

    public static String getRequiredStringAttribute(Element element, String attributeName) {
        if (element.hasAttribute(attributeName)) {
            return element.getAttribute(attributeName);
        }
        throw new IllegalArgumentException("Attribute " + attributeName + " not provided");
    }

    public static TypeName getRequiredTypeAttribute(Element element, String attributeName) {
        TypeName type = getOptionalTypeAttribute(element, attributeName);
        if (type == null) {
            throw new IllegalArgumentException("Attribute " + attributeName + " not provided");
        }
        return type;
    }

    public static TypeName getOptionalTypeAttribute(Element element, String attributeName, TypeName defaultValue) {
        String typeAttr = getOptionalStringAttribute(element, attributeName, null);
        TypeName type = defaultValue;
        if (typeAttr != null) {
            type = new TypeName(typeAttr);
        }
        return type;
    }

    public static TypeName getOptionalTypeAttribute(Element element, String attributeName) {
        return getOptionalTypeAttribute(element, attributeName, null);
    }

    public static Optional<TypeName> getOptionalType(Element element, String attributeName) {
        return getOptionalString(element, attributeName).map(TypeName::new);
    }

    public static boolean getOptionalBooleanAttribute(Element element, String attributeName, boolean defaultValue) {
        return getOptionalBoolean(element, attributeName).orElse(defaultValue);
    }

    public static Optional<Boolean> getOptionalBoolean(Element element, String attributeName) {
        return getOptionalString(element, attributeName).map(Boolean::valueOf);
    }

    public static String getOptionalStringAttribute(Element element, String attributeName, String defaultValue) {
        return getOptionalString(element, attributeName).orElse(defaultValue);
    }

    public static Optional<String> getOptionalString(Element element, String attributeName) {
        if (element.hasAttribute(attributeName)) {
            return Optional.of(element.getAttribute(attributeName));
        } else {
            return Optional.empty();
        }
    }

    public static Optional<String> getOptionalTextOfChild(Element element, String childElementName) {
        Iterator<Element> children = childrenByName(element, childElementName::equals).iterator();
        if (children.hasNext()) {
            return Optional.ofNullable(children.next().getFirstChild().getTextContent());
        }
        return Optional.empty();
    }

    public static Iterable<Element> childrenByName(final Element element, final Predicate<String> matcher) {
        return () -> childByName(element, matcher);
    }

    public static Iterator<Element> childByName(final Element element, final Predicate<String> matcher) {
        return new Iterator<Element>() {
            private int i = -1;
            private final NodeList childNodes = element.getChildNodes();
            private int nextIndex = -1;

            @Override
            public boolean hasNext() {
                if (nextIndex == i) nextIndex = findNext();
                return nextIndex > i;
            }

            @Override
            public Element next() {
                if (nextIndex == i) nextIndex = findNext();
                if (nextIndex < i) throw new NoSuchElementException("There are no more matching elements");
                i = nextIndex;

                return (Element) childNodes.item(i);
            }

            private int findNext() {
                int next = i;
                while (next < childNodes.getLength()) {
                    next++;
                    if (childNodes.item(next) instanceof Element) {
                        Element e = (Element) childNodes.item(next);
                        if (matcher.test(e.getNodeName())) {
                            return next;
                        }
                    }
                }
                return i - 1;
            }

            @Override
            public void remove() {
                throw new UnsupportedOperationException();
            }
        };
    }

    public static <E> void forEach(Iterator<E> iterator, Closure<E> closure) {
        while (iterator.hasNext()) {
            closure.call(iterator.next());
        }
    }

    public interface Closure<E> {
        void call(E e);
    }
}
